@@ -38,6 +38,9 @@
 
 #include <libgnomeprint/private/gnome-print-private.h>
 #include <libgnomeprint/gnome-print-meta.h>
+#include <libgnomeprint/gnome-print-config.h>
+#include <libgnomeprint/private/gpa-node.h>
+#include <libgnomeprint/private/gnome-print-config-private.h>
 
 #include <libgnomeprintui/gnome-print-i18n.h>
 #include <libgnomeprintui/gnome-print-job-preview.h>
@@ -140,10 +143,14 @@ struct _GnomePrintJobPreview {
 	/* Our GnomePrintJob */
 	GnomePrintJob *job;
 	gulong notify_id;
-	
+
+	GPANode *node_paper_size, *node_page_orient;
+	gulong handler_paper_size, handler_page_orient;
+
 	GtkWidget *page_entry;
 	GtkWidget *last;
 	GnomeCanvas *canvas;
+	GnomePrintConfig *config;
 
 	guint current_page, current_offset;
 
@@ -437,6 +444,33 @@ gnome_print_job_preview_hide_page (GnomePrintJobPreview *jp, guint n)
 	gnome_canvas_item_hide (p.group);
 }
 
+static void
+gnome_print_job_preview_update_page (GnomePrintJobPreview *jp, GnomePrintJobPreviewPage *p)
+{
+	GnomeCanvasItem *group;
+	gdouble t[6];
+
+	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+	g_return_if_fail (p);
+
+	if (p->n >= jp->selection->len) {
+		gnome_canvas_item_hide (p->group);
+		return;
+	}
+	gnome_canvas_item_show (p->group);
+
+	g_object_set (G_OBJECT (p->page_bg),
+			"x2", jp->paw + 3., "y2", jp->pah + 3., NULL);
+	g_object_set (G_OBJECT (p->page_fg), "x2", jp->paw, "y2", jp->pah, NULL);
+	g_object_get (G_OBJECT (p->preview), "group", &group, NULL);
+	t[0] =  1.; t[1] =  0.; t[2] =  0.;
+	t[3] = -1.; t[4] =  0.; t[5] =  jp->pah;
+	gnome_canvas_item_affine_absolute (group, t);
+	gnome_print_preview_reset (p->preview);
+	gnome_print_job_render_page (jp->job, GNOME_PRINT_CONTEXT (p->preview),
+			p->n, TRUE);
+}
+
 static void
 gnome_print_job_preview_show_page (GnomePrintJobPreview *jp, guint n,
 		guint page)
@@ -448,13 +482,8 @@ gnome_print_job_preview_show_page (GnomePrintJobPreview *jp, guint n,
 	g_return_if_fail (n < jp->pages->len);
 
 	p = &(g_array_index (jp->pages, GnomePrintJobPreviewPage, n));
-
-	gnome_print_job_render_page (jp->job, GNOME_PRINT_CONTEXT (p->preview),
-			page, TRUE);
 	p->n = page;
-
-	if (!gnome_print_job_preview_page_is_visible (jp, n))
-		gnome_canvas_item_show (p->group);
+	gnome_print_job_preview_update_page (jp, p);
 }
 
 static void
@@ -478,7 +507,7 @@ gnome_print_job_preview_show_pages (GnomePrintJobPreview *jp, guint page)
 static void
 gnome_print_job_preview_goto_page (GnomePrintJobPreview *jp, guint page)
 {
-	guchar c[32];
+	gchar c[32];
 	guint i;
 
 	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
@@ -829,58 +858,14 @@ gnome_print_job_preview_redo (GnomePrintJobPreview *jp)
 static void
 gnome_print_job_preview_zoom (GnomePrintJobPreview *jp, gdouble factor)
 {
-	gdouble	     zoom, xdpi, ydpi;
-	int          w, h, w_mm, h_mm;
-	GdkScreen   *screen;
+	gdouble	     zoom;
+	int          w, h;
 
 	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
 
 	/* No pages -> no zoom. */
 	if (!jp->nx || !jp->ny) return;
 
-	screen = gtk_widget_get_screen (GTK_WIDGET (jp->canvas)),
-#if 0
-       /* there is no per monitor physical size info available for now */
-	n = gdk_screen_get_n_monitors (screen);
-	for (i = 0; i < n; i++) {
-		gdk_screen_get_monitor_geometry (screen, i, &monitor);
-		if (x >= monitor.x && x <= monitor.x + monitor.width &&
-		    y >= monitor.y && y <= monitor.y + monitor.height) {
-			break;
-		}
-	}
-#endif
-	/*
-	 * From xdpyinfo.c:
-	 *
-	 * there are 2.54 centimeters to an inch; so there are 25.4 millimeters.
-	 *
-	 *     dpi = N pixels / (M millimeters / (25.4 millimeters / 1 inch))
-	 *         = N pixels / (M inch / 25.4)
-	 *         = N * 25.4 pixels / M inch
-	 */
-	w_mm = gdk_screen_get_width_mm (screen);
-	xdpi = -1.;
-	if (w_mm > 0) {
-		w    = gdk_screen_get_width (screen);
-		xdpi = (w * 25.4) / (gdouble) w_mm;
-	} 
-	if (xdpi < 30. || 600. < xdpi) {
-		g_warning ("Invalid the x-resolution for the screen, assuming 96dpi");
-		xdpi = 96.;
-	}
-
-	h_mm = gdk_screen_get_height_mm (screen);
-	ydpi = -1.;
-	if (h_mm > 0) {
-		h    = gdk_screen_get_height (screen);
-		ydpi = (h * 25.4) / (gdouble) h_mm;
-	}
-	if (ydpi < 30. || 600. < ydpi) {
-		g_warning ("Invalid the y-resolution for the screen, assuming 96dpi");
-		ydpi = 96.;
-	}
-
 	if (factor <= 0.) {
 		gint width = GTK_WIDGET (jp->canvas)->allocation.width;
 		gint height = GTK_WIDGET (jp->canvas)->allocation.height;
@@ -889,7 +874,8 @@ gnome_print_job_preview_zoom (GnomePrintJobPreview *jp, gdouble factor)
 
 		zoom = MIN (zoomx, zoomy);
 	} else
-		zoom = jp->zoom_factor * factor * (xdpi + ydpi) / (72. * 2.);
+		zoom = jp->zoom_factor * factor;
+
 	jp->zoom_factor = CLAMP (zoom, GPMP_ZOOM_MIN, GPMP_ZOOM_MAX);
 	gnome_canvas_set_pixels_per_unit (jp->canvas, jp->zoom_factor);
 
@@ -901,7 +887,7 @@ gnome_print_job_preview_zoom (GnomePrintJobPreview *jp, gdouble factor)
 static guint
 gnome_print_job_preview_get_page_at (GnomePrintJobPreview *jp, guint x, guint y)
 {
-	guint ox, oy;
+	gint ox, oy;
 	guint row, col;
 
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp), 0);
@@ -1494,6 +1480,7 @@ gnome_print_job_preview_nx_and_ny_changed (GnomePrintJobPreview *jp)
 	while (jp->pages->len > MIN (jp->nx * jp->ny, jp->selection->len)) {
 		p = g_array_index (jp->pages, GnomePrintJobPreviewPage, 0);
 		gtk_object_destroy (GTK_OBJECT (p.group));
+		g_object_unref (G_OBJECT (p.preview));
 		g_array_remove_index (jp->pages, 0);
 	}
 
@@ -1512,52 +1499,35 @@ gnome_print_job_preview_nx_and_ny_changed (GnomePrintJobPreview *jp)
 		art_affine_multiply (transform, jp->pa2ly, transform);
 
 		while (jp->pages->len < jp->nx * jp->ny) {
-			GnomeCanvasItem *group;
-
-			p.group = gnome_canvas_item_new (
-				gnome_canvas_root (jp->canvas),
-				GNOME_TYPE_CANVAS_GROUP,
-				"x", 0., "y", 0., NULL);
+			p.group = gnome_canvas_item_new (gnome_canvas_root (jp->canvas),
+				GNOME_TYPE_CANVAS_GROUP, NULL);
 			gnome_canvas_item_hide (p.group);
-			p.page_fg = gnome_canvas_item_new (
-				GNOME_CANVAS_GROUP (p.group),
-				GNOME_TYPE_CANVAS_RECT, "x1", 0.0, "y1", 0.0,
-				"x2", (gdouble) jp->paw,
-				"y2", (gdouble) jp->pah,
-				"fill_color", "white",
-				"outline_color_rgba", c,
-				"width_pixels", 1, NULL);
+			p.page_fg = gnome_canvas_item_new (GNOME_CANVAS_GROUP (p.group),
+					GNOME_TYPE_CANVAS_RECT, "fill_color", "white",
+					"outline_color_rgba", c, "width_pixels", 1, NULL);
 			gnome_canvas_item_lower_to_bottom (p.page_fg);
-			p.page_bg = gnome_canvas_item_new (
-				GNOME_CANVAS_GROUP (p.group),
-				GNOME_TYPE_CANVAS_RECT, "x1", 3.0, "y1", 3.0,
-				"x2", (gdouble) jp->paw + 3,
-				"y2", (gdouble) jp->pah + 3,
-				"fill_color", "black",
-				"outline_color", "black", NULL);
+			p.page_bg = gnome_canvas_item_new (GNOME_CANVAS_GROUP (p.group),
+					GNOME_TYPE_CANVAS_RECT, "x1", 3.0, "y1", 3.0,
+					"fill_color", "black", "outline_color", "black", NULL);
 			gnome_canvas_item_lower_to_bottom (p.page_bg);
 
 			/* Create the group that holds the preview. */
-			group = gnome_canvas_item_new (
-				GNOME_CANVAS_GROUP (p.group),
-				GNOME_TYPE_CANVAS_GROUP,
-				"x", 0., "y", 0., NULL);
-			gnome_canvas_item_affine_absolute (group, transform); 
 			p.preview = g_object_new (GNOME_TYPE_PRINT_PREVIEW,
-				"group", group,
+				"group", gnome_canvas_item_new (
+					GNOME_CANVAS_GROUP (p.group), GNOME_TYPE_CANVAS_GROUP, NULL),
 				"theme_compliance", jp->theme_compliance,
 				NULL);
-			g_array_append_val (jp->pages, p);
+			if (!jp->pages || !jp->pages->len)
+				p.n = 0;
+			else {
+				GnomePrintJobPreviewPage pp;
 
-			if (jp->pages->len == 1)
-				gnome_print_job_preview_show_page (jp, 0, 0);
-			else if (jp->pages->len > 1 &&
-					gnome_print_job_preview_page_is_visible (jp, jp->pages->len - 2)) {
-				p = g_array_index (jp->pages, GnomePrintJobPreviewPage,
-						jp->pages->len - 2);
-				if (p.n + 1 < jp->selection->len)
-					gnome_print_job_preview_show_page (jp, jp->pages->len - 1, p.n + 1);
+				pp = g_array_index (jp->pages, GnomePrintJobPreviewPage,
+						jp->pages->len - 1);
+				p.n = pp.n + 1;
 			}
+			g_array_append_val (jp->pages, p);
+			gnome_print_job_preview_update_page (jp, &p);
 		}
 	}
 
@@ -2071,32 +2041,13 @@ gnome_print_job_preview_check_number_of_pages (GnomePrintJobPreview *jp)
 	}
 }
 
-static void
-on_job_notify (GObject *object, GParamSpec *pspec, GnomePrintJobPreview *jp)
-{
-	guint i;
-
-	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
-
-	/* Check number of pages. */
-	gnome_print_job_preview_check_number_of_pages (jp);
-
-	/* Because we don't know if the content has changed, redraw everything. */
-	for (i = 0; i < jp->pages->len; i++) {
-		GnomePrintJobPreviewPage p;
-
-		if (!gnome_print_job_preview_page_is_visible (jp, i)) continue;
-		p = g_array_index (jp->pages, GnomePrintJobPreviewPage, i);
-		gnome_print_job_preview_show_page (jp, i, p.n);
-	}
-}
-
 static void
 gnome_print_job_preview_width_height_changed (GnomePrintJobPreview *jp)
 {
 	GdkDisplay *d = gtk_widget_get_display (GTK_WIDGET (jp));
 	GdkScreen *s = gdk_display_get_screen (d, 0);
 	gint w, h;
+	guint i;
 	GnomeCanvasPoints *points;
 	GnomeCanvasItem *item;
 	GdkGeometry hints;
@@ -2143,6 +2094,140 @@ gnome_print_job_preview_width_height_changed (GnomePrintJobPreview *jp)
 	gtk_window_set_default_size (GTK_WINDOW (jp),
 			MIN (jp->paw + PAGE_PAD * 3, w * 3 / 4),
 			MIN (jp->pah + PAGE_PAD * 3, h * 3 / 4));
+
+	for (i = 0; i < (jp->pages ? jp->pages->len : 0); i++) {
+		GnomePrintJobPreviewPage p;
+
+		p = g_array_index (jp->pages, GnomePrintJobPreviewPage, i);
+		gnome_print_job_preview_update_page (jp, &p);
+	}
+}
+
+static void
+gnome_print_job_preview_set_width (GnomePrintJobPreview *jp, gdouble width)
+{
+	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+
+	if (jp->paw == width)
+		return;
+	jp->paw = width;
+	gnome_print_job_preview_width_height_changed (jp);
+}
+
+static void
+gnome_print_job_preview_set_height (GnomePrintJobPreview *jp, gdouble height)
+{
+	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+
+	if (jp->pah == height)
+		return;
+	jp->pah = height;
+	gnome_print_job_preview_width_height_changed (jp);
+}
+
+static void
+gnome_print_job_preview_check_paper_size (GnomePrintJobPreview *jp)
+{
+	gdouble a_page[6];
+	const GnomePrintUnit *unit = NULL;
+	ArtPoint p1, p2;
+
+	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+
+	gnome_print_config_get_length (jp->config,
+			(const guchar *) GNOME_PRINT_KEY_PAPER_WIDTH, &p1.x, &unit);
+	gnome_print_convert_distance (&p1.x, unit, GNOME_PRINT_PS_UNIT);
+	gnome_print_config_get_length (jp->config,
+			(const guchar *) GNOME_PRINT_KEY_PAPER_HEIGHT, &p1.y, &unit);
+	gnome_print_convert_distance (&p1.y, unit, GNOME_PRINT_PS_UNIT);
+	gnome_print_config_get_transform (jp->config,
+			(const guchar *) GNOME_PRINT_KEY_PAGE_ORIENTATION_MATRIX, a_page);
+	art_affine_point (&p2, &p1, a_page);
+	gnome_print_job_preview_set_width (jp, fabs (p2.x));
+	gnome_print_job_preview_set_height (jp, fabs (p2.y));
+}
+
+static void
+on_paper_size_modified (GPANode *node, guint flags, GnomePrintJobPreview *jp)
+{
+	gnome_print_job_preview_check_paper_size (jp);
+}
+
+static void
+on_page_orient_modified (GPANode *node, guint flags, GnomePrintJobPreview *jp)
+{
+	gnome_print_job_preview_check_paper_size (jp);
+}
+
+static void
+gnome_print_job_preview_set_config (GnomePrintJobPreview *jp,
+		GnomePrintConfig *config)
+{
+	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+	g_return_if_fail (!config || GNOME_IS_PRINT_CONFIG (config));
+
+	if (jp->config == config)
+		return;
+	if (jp->node_paper_size) {
+		if (jp->handler_paper_size) {
+			g_signal_handler_disconnect (G_OBJECT (jp->node_paper_size),
+					jp->handler_paper_size);
+			jp->handler_paper_size = 0;
+		}
+		jp->node_paper_size = NULL;
+	}
+	if (jp->node_page_orient) {
+		if (jp->handler_page_orient) {
+			g_signal_handler_disconnect (G_OBJECT (jp->node_page_orient),
+					jp->handler_page_orient);
+			jp->handler_page_orient = 0;
+		}
+		jp->node_page_orient = NULL;
+	}
+	if (jp->config)
+		g_object_unref (G_OBJECT (jp->config));
+	jp->config = config;
+	if (!jp->config)
+		return;
+	g_object_ref (G_OBJECT (jp->config));
+  jp->node_paper_size = gpa_node_get_child_from_path (
+			GNOME_PRINT_CONFIG_NODE (jp->config),
+			(const guchar *) GNOME_PRINT_KEY_PAPER_SIZE);
+  jp->handler_paper_size = g_signal_connect (G_OBJECT (jp->node_paper_size),
+			"modified", G_CALLBACK (on_paper_size_modified), jp);
+	jp->node_page_orient = gpa_node_get_child_from_path (
+			GNOME_PRINT_CONFIG_NODE (jp->config),
+			(const guchar *) GNOME_PRINT_KEY_PAGE_ORIENTATION);
+	jp->handler_page_orient = g_signal_connect (G_OBJECT (jp->node_page_orient),
+			"modified", G_CALLBACK (on_page_orient_modified), jp);
+	gnome_print_job_preview_check_paper_size (jp);
+}
+
+static void
+on_job_notify (GObject *object, GParamSpec *pspec, GnomePrintJobPreview *jp)
+{
+	guint i;
+
+	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+
+	if (!strcmp (pspec->name, "config")) {
+		GnomePrintConfig *config;
+
+		g_object_get (object, "config", &config, NULL);
+		gnome_print_job_preview_set_config (jp, config);
+	}
+
+	/* Check number of pages. */
+	gnome_print_job_preview_check_number_of_pages (jp);
+
+	/* Because we don't know if the content has changed, redraw everything. */
+	for (i = 0; i < jp->pages->len; i++) {
+		GnomePrintJobPreviewPage p;
+
+		if (!gnome_print_job_preview_page_is_visible (jp, i)) continue;
+		p = g_array_index (jp->pages, GnomePrintJobPreviewPage, i);
+		gnome_print_job_preview_show_page (jp, i, p.n);
+	}
 }
 
 static void
@@ -2204,23 +2289,25 @@ gnome_print_job_preview_parse_layout (GnomePrintJobPreview *jp)
 		}
 		gnome_print_layout_data_free (lyd);
 	}
-	if ((jp->paw != w) || (jp->pah != h)) {
-		jp->paw = w;
-		jp->pah = h;
-		gnome_print_job_preview_width_height_changed (jp);
-	}
+	gnome_print_job_preview_set_width (jp, w);
+	gnome_print_job_preview_set_height (jp, h);
 }
 
 static void
 gnome_print_job_preview_set_job (GnomePrintJobPreview *jp, GnomePrintJob *job)
 {
+	GnomePrintConfig *config = NULL;
+
 	g_return_if_fail (GNOME_IS_PRINT_JOB_PREVIEW (jp));
+	g_return_if_fail (!job || GNOME_IS_PRINT_JOB (job));
 
 	if (jp->job) {
-		g_signal_handler_disconnect (G_OBJECT (jp->job), jp->notify_id);
+		if (jp->notify_id) {
+			g_signal_handler_disconnect (G_OBJECT (jp->job), jp->notify_id);
+			jp->notify_id = 0;
+		}
 		g_object_unref (G_OBJECT (jp->job));
 		jp->job = NULL;
-		jp->notify_id = 0;
 		g_array_set_size (jp->selection, 0);
 	}
 	if (!job) return;
@@ -2231,52 +2318,9 @@ gnome_print_job_preview_set_job (GnomePrintJobPreview *jp, GnomePrintJob *job)
 			G_CALLBACK (on_job_notify), jp);
 	gnome_print_job_preview_parse_layout (jp);
 	gnome_print_job_preview_check_number_of_pages (jp);
-}
 
-static void
-gnome_print_job_preview_destroy (GtkObject *object)
-{
-	GnomePrintJobPreview *jp = GNOME_PRINT_JOB_PREVIEW (object);
-
-	if (jp->clipboard) {
-		g_object_unref (G_OBJECT (jp->clipboard));
-		jp->clipboard = NULL;
-	}
-
-	if (jp->undo) {
-		gnome_print_job_preview_clear_undo (jp);
-		g_array_free (jp->undo, TRUE);
-		jp->undo = NULL;
-	}
-	if (jp->redo) {
-		gnome_print_job_preview_clear_redo (jp);
-		g_array_free (jp->redo, TRUE);
-		jp->redo = NULL;
-	}
-
-	gnome_print_job_preview_set_job (jp, NULL);
-
-	if (jp->pages != NULL) {
-		g_array_free (jp->pages, TRUE);
-		jp->pages = NULL;
-	}
-
-	if (jp->selection != NULL) {
-		g_array_free (jp->selection, TRUE);
-		jp->selection = NULL;
-	}
-
-	if (jp->event) {
-		gdk_event_free (jp->event);
-		jp->event = NULL;
-	}
-
-	if (jp->ui_manager) {
-		g_object_unref (G_OBJECT (jp->ui_manager));
-		jp->ui_manager = NULL;
-	}
-
-	GTK_OBJECT_CLASS (parent_class)->destroy (object);
+	g_object_get (G_OBJECT (jp->job), "config", &config, NULL);
+	gnome_print_job_preview_set_config (jp, config);
 }
 
 enum {
@@ -2340,18 +2384,64 @@ gnome_print_job_preview_set_property (GObject *object, guint n,
 	}
 }
 
+static void
+gnome_print_job_preview_finalize (GObject *object)
+{
+	GnomePrintJobPreview *jp = GNOME_PRINT_JOB_PREVIEW (object);
+
+	gnome_print_job_preview_set_config (jp, NULL);
+	gnome_print_job_preview_set_job (jp, NULL);
+
+	if (jp->selection != NULL) {
+		g_array_free (jp->selection, TRUE);
+		jp->selection = NULL;
+	}
+
+	if (jp->clipboard) {
+		g_object_unref (G_OBJECT (jp->clipboard));
+		jp->clipboard = NULL;
+	}
+
+	if (jp->undo) {
+		gnome_print_job_preview_clear_undo (jp);
+		g_array_free (jp->undo, TRUE);
+		jp->undo = NULL;
+	}
+	if (jp->redo) {
+		gnome_print_job_preview_clear_redo (jp);
+		g_array_free (jp->redo, TRUE);
+		jp->redo = NULL;
+	}
+
+	if (jp->pages != NULL) {
+		unsigned i;
+		for (i = jp->pages->len; i-- > 0 ; )
+			g_object_unref (g_array_index (jp->pages, GnomePrintJobPreviewPage, i).preview);
+		g_array_free (jp->pages, TRUE);
+		jp->pages = NULL;
+	}
+
+	if (jp->event) {
+		gdk_event_free (jp->event);
+		jp->event = NULL;
+	}
+
+	if (jp->ui_manager) {
+		g_object_unref (G_OBJECT (jp->ui_manager));
+		jp->ui_manager = NULL;
+	}
+
+	G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gnome_print_job_preview_class_init (GnomePrintJobPreviewClass *klass)
 {
-	GtkObjectClass *object_class;
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
-	object_class = (GtkObjectClass *) klass;
-
 	parent_class = gtk_type_class (GTK_TYPE_WINDOW);
 
-	object_class->destroy = gnome_print_job_preview_destroy;
-
+	gobject_class->finalize     = gnome_print_job_preview_finalize;
 	gobject_class->set_property = gnome_print_job_preview_set_property;
 	gobject_class->get_property = gnome_print_job_preview_get_property;
 
@@ -2430,11 +2520,11 @@ on_focus_out_event (GtkWidget *widget, GdkEventFocus *event)
 static void
 gnome_print_job_preview_init (GnomePrintJobPreview *jp)
 {
-	GtkWidget *vbox, *tb, *l, *status;
+	GtkWidget *vbox, *tb = NULL, *l, *status;
 	const gchar *env_theme_variable;
 	gchar *text, *ui_file;
 	AtkObject *atko;
-	GList *action_groups;
+	GList *action_groups, *actions, *g_ptr, *a_ptr;
 	GError *e = NULL;
 
 	jp->ui_manager = g_object_new (GTK_TYPE_UI_MANAGER, NULL);
@@ -2490,10 +2580,10 @@ gnome_print_job_preview_init (GnomePrintJobPreview *jp)
 
 	/* Navigation */
 	jp->nav.group = gtk_action_group_new ("navigation");
-	jp->nav.f = gtk_action_new ("first", _("First"), _("Shows the first page"), GTK_STOCK_GOTO_FIRST);
-	jp->nav.p = gtk_action_new ("previous", _("Previous"),  _("Shows previous page"), GTK_STOCK_GO_BACK);
-	jp->nav.n = gtk_action_new ("next", _("Next"), _("Shows the next page"), GTK_STOCK_GO_FORWARD);
-	jp->nav.l = gtk_action_new ("last", _("Last"), _("Shows the last page"), GTK_STOCK_GOTO_LAST);
+	jp->nav.f = gtk_action_new ("first", _("First"), _("Show the first page"), GTK_STOCK_GOTO_FIRST);
+	jp->nav.p = gtk_action_new ("previous", _("Previous"),  _("Show previous page"), GTK_STOCK_GO_BACK);
+	jp->nav.n = gtk_action_new ("next", _("Next"), _("Show the next page"), GTK_STOCK_GO_FORWARD);
+	jp->nav.l = gtk_action_new ("last", _("Last"), _("Show the last page"), GTK_STOCK_GOTO_LAST);
 	gtk_action_group_add_action_with_accel (jp->nav.group, jp->nav.f, NULL);
 	gtk_action_group_add_action_with_accel (jp->nav.group, jp->nav.p, NULL);
 	gtk_action_group_add_action_with_accel (jp->nav.group, jp->nav.n, NULL);
@@ -2503,10 +2593,10 @@ gnome_print_job_preview_init (GnomePrintJobPreview *jp)
 	/* Zoom */
 	jp->zoom.group = gtk_action_group_new ("zoom");
 	/* xgettext:no-c-format */
-	jp->zoom.z1 = gtk_action_new ("zoom_100", _("100%"), _("Zooms 1:1"), GTK_STOCK_ZOOM_100);
-	jp->zoom.zf = gtk_action_new ("zoom_fit", _("Zoom to fit"), _("Zooms to fit the whole page"), GTK_STOCK_ZOOM_FIT);
-	jp->zoom.zi = gtk_action_new ("zoom_in", _("Zoom in"), _("Zooms the page in"), GTK_STOCK_ZOOM_IN);
-	jp->zoom.zo = gtk_action_new ("zoom_out", _("Zoom out"), _("Zooms the page out"), GTK_STOCK_ZOOM_OUT);
+	jp->zoom.z1 = gtk_action_new ("zoom_100", _("100%"), _("Zoom 1:1"), GTK_STOCK_ZOOM_100);
+	jp->zoom.zf = gtk_action_new ("zoom_fit", _("Zoom to fit"), _("Zoom to fit the whole page"), GTK_STOCK_ZOOM_FIT);
+	jp->zoom.zi = gtk_action_new ("zoom_in", _("Zoom in"), _("Zoom the page in"), GTK_STOCK_ZOOM_IN);
+	jp->zoom.zo = gtk_action_new ("zoom_out", _("Zoom out"), _("Zoom the page out"), GTK_STOCK_ZOOM_OUT);
 	gtk_action_group_add_action_with_accel (jp->zoom.group, jp->zoom.z1, NULL);
 	gtk_action_group_add_action_with_accel (jp->zoom.group, jp->zoom.zf, NULL);
 	gtk_action_group_add_action_with_accel (jp->zoom.group, jp->zoom.zi, NULL);
@@ -2597,24 +2687,21 @@ gnome_print_job_preview_init (GnomePrintJobPreview *jp)
 	gtk_widget_show_all (status);
 
 	action_groups = gtk_ui_manager_get_action_groups (jp->ui_manager);
-        while (action_groups) {
-		GtkActionGroup *ag = action_groups->data;
-		GList *actions = gtk_action_group_list_actions (ag);
+        for (g_ptr = action_groups; g_ptr != NULL ; g_ptr = g_ptr->next) {
+		GtkActionGroup *ag = g_ptr->data;
 
-		while (actions) {
-			GtkAction *a = actions->data;
+		actions = gtk_action_group_list_actions (ag);
+		for (a_ptr = actions ; a_ptr != NULL ; a_ptr = a_ptr->next) {
+			GtkAction *a = a_ptr->data;
 
 			gtk_action_connect_accelerator (a);
 			if (GTK_IS_TOGGLE_ACTION (a))
 				g_signal_connect (G_OBJECT (a), "toggled", G_CALLBACK (on_action_toggled), jp);
 			else
 				g_signal_connect (G_OBJECT (a), "activate", G_CALLBACK (on_action_activate), jp);
-			actions = actions->next;
 		}
 		g_list_free (actions);
-		action_groups = action_groups->next;
 	}
-	g_list_free (action_groups);
 
 	/* Connect the signals */
 	g_signal_connect (G_OBJECT (jp->canvas), "button_press_event", G_CALLBACK (on_canvas_button_press_event), jp);
@@ -2680,7 +2767,7 @@ gnome_print_job_preview_new (GnomePrintJob *job, const guchar *title)
 	jp = g_object_new (GNOME_TYPE_PRINT_JOB_PREVIEW, "job", job, NULL);
 
 	gtk_window_set_title (GTK_WINDOW (jp),
-		title ? title : (const guchar *) _("Gnome Print Preview"));
+		title ? (gchar const *)title : _("Gnome Print Preview"));
 
 	/* Start monitoring the clipboard. */
 	clipboard = gtk_clipboard_get_for_display (
